unit FPropertyPersist;

{ FPropertyPersist.pas
  FPropertyPersist.dfm

Copyright (C) 2001  Richard T. Weeks

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Commercial development licenses for all files in the Registry Persistence
Package can be obtained from the author for US$10.  This includes
    RegistryPersist.pas
    FRegistryBrowser.pas
    FRegistryBrowser.dfm
    FPropertyPersist.pas
    FPropertyPersist.dfm
    FSimpleInput.pas
    FSimpleInput.dfm
    RegPersistDsgn.pas

e-mail addresses:
Richard T. Weeks - rtweeks@acm.org
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  CheckLst, ExtCtrls, StdCtrls, DsgnIntf, RegistryPersist, ComCtrls;

type
  TPropertyPersistForm = class(TForm)
    Components: TListBox;
    Splitter1: TSplitter;
    Properties: TCheckListBox;
    procedure ComponentsClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure PropertiesClickCheck(Sender: TObject);
  private
    FDesigner: IFormDesigner;
    FRegPersist: TRegistryPersist;
    FCurComponent: TComponent;
    FCurObj: TPersistent;
    procedure ListProperties;
    procedure AddComponentName(const ComponentName: String);
    { Private declarations }
  public
    procedure Execute(ADesigner: IFormDesigner; ARegPersist: TRegistryPersist);
    { Public declarations }
  end;

var
  PropertyPersistForm: TPropertyPersistForm;

implementation

uses TypInfo;

{$R *.DFM}

{ TPropertyPersistForm }

procedure TPropertyPersistForm.AddComponentName(
  const ComponentName: String);
var
  Component: TComponent;
  TopNode: TTreeNode;
begin
  Component := FDesigner.GetComponent(ComponentName);
  Components.Items.AddObject(ComponentName, Component);
end;

procedure TPropertyPersistForm.Execute(ADesigner: IFormDesigner;
  ARegPersist: TRegistryPersist);
begin
  FDesigner := ADesigner;
  FRegPersist := ARegPersist;

  Show;
end;

procedure TPropertyPersistForm.ComponentsClick(Sender: TObject);
begin
  ListProperties;
end;

procedure TPropertyPersistForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TPropertyPersistForm.FormActivate(Sender: TObject);
var
  Current: TObject;
  Root: TComponent;
begin
  if Components.ItemIndex > 0 then
    Current := Components.Items.Objects[Components.ItemIndex]
  else
    Current := nil;

  Components.Items.BeginUpdate;
  try
    Components.Items.Clear;

    FDesigner.GetComponentNames(GetTypeData(TypeInfo(TComponent)),
      AddComponentName);

    Root := FDesigner.GetRoot;
    if Root <> nil then begin
      Components.Items.AddObject(Root.Name, Root);
    end;
  finally
    Components.Items.EndUpdate;
  end;

  if Current <> nil then
    Components.ItemIndex := Components.Items.IndexOfObject(Current);

  ListProperties;
end;

procedure TPropertyPersistForm.ListProperties;
var
  TypeData, PropType: PTypeData;
  PropList: array of PPropInfo;
  I, J, LastIndex: Integer;
begin
  Properties.Items.BeginUpdate;
  try
    Properties.Items.Clear;

    if Components.ItemIndex < 0 then
      Exit;

    FCurComponent := TComponent(Components.Items.Objects[Components.ItemIndex]);
    TypeData := GetTypeData(FCurComponent.ClassInfo);
    SetLength(PropList, TypeData^.PropCount);
    SetLength(PropList, GetPropList(FCurComponent.ClassInfo, [tkInteger, tkChar,
      tkEnumeration, tkFloat, tkString, tkSet, tkWChar, tkLString, tkWString,
      tkClass],
      PPropList(PropList)));
    for I := 0 to Length(PropList) - 1 do begin
      if PropList[I]^.PropType^^.Kind = tkClass then begin
        PropType := GetTypeData(PropList[I]^.PropType^);
        if not PropType^.ClassType.InheritsFrom(TStrings) then
          Continue;
      end;

      Properties.Items.Add(PropList[I]^.Name);
      for J := 0 to FRegPersist.Properties.Count - 1 do
        with FRegPersist.Properties[J] do
          if (FCurComponent = Component) and (PropList[I]^.Name = PropName) then
      begin
        LastIndex := Properties.Items.Count - 1;
        Properties.State[LastIndex] := cbChecked;
        Properties.Items.Objects[LastIndex] := FRegPersist.Properties[J];
        Break;
      end;
    end;
  finally
    Properties.Items.EndUpdate;
  end;
end;

procedure TPropertyPersistForm.PropertiesClickCheck(Sender: TObject);
begin
  if Properties.ItemIndex < 0 then
    Exit;

  if Properties.State[Properties.ItemIndex] = cbChecked then begin
    // Create a TPropertyPersist for this property
    with FRegPersist.Properties.Add do begin
      Component := FCurComponent;
      PropName := Properties.Items[Properties.ItemIndex];
    end;
  end else begin
    // Remove the TPropertyPersist associated with this property
    Properties.Items.Objects[Properties.ItemIndex].Free;
    Properties.Items.Objects[Properties.ItemIndex] := nil;
  end;
end;

end.
