unit SimpleInput;

{ SimpleInput.pas
  SimpleInput.dfm

Copyright (C) 2001  Richard T. Weeks

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

Commercial development licenses for all files in the Registry Persistence
Package can be obtained from the author for US$10.  This includes
    RegistryPersist.pas
    FRegistryBrowser.pas
    FRegistryBrowser.dfm
    FPropertyPersist.pas
    FPropertyPersist.dfm
    FSimpleInput.pas
    FSimpleInput.dfm
    RegPersistDsgn.pas

e-mail addresses:
Richard T. Weeks - rtweeks@acm.org
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type
  TValidationProc = procedure(const Value: String; var Valid: Boolean) of object;
  TSimpleInputForm = class(TForm)
    MessageLabel: TLabel;
    ValueEdit: TEdit;
    Button1: TButton;
    Button2: TButton;
    procedure Button1Click(Sender: TObject);
  private
    FOnValidate: TValidationProc;
    procedure SetOnValidate(const Value: TValidationProc);
    { Private declarations }
  public
    property OnValidate: TValidationProc read FOnValidate write SetOnValidate;
    { Public declarations }
  end;

function GetSimpleInput(const Title, Message: String;
  var Value: String): Boolean;
function GetValidatedInput(const Title, Message: String; var Value: String;
  Validator: TValidationProc): Boolean;

implementation

{$R *.DFM}

const
  SInvalidValue = 'Invalid Value';

function GetSimpleInput(const Title, Message: String; var Value: String): Boolean;
begin
  Result := GetValidatedInput(Title, Message, Value, nil);
end;

function GetValidatedInput(const Title, Message: String; var Value: String;
  Validator: TValidationProc): Boolean;
var
  Form: TSimpleInputForm;
begin
  Form := TSimpleInputForm.Create(nil);
  try
    with Form do begin
      Caption := Title;
      MessageLabel.Caption := Message;
      ValueEdit.Text := Value;
      Result := ShowModal = mrOk;
      if Result then
        Value := ValueEdit.Text;
    end;
  finally
    Form.Free;
  end;
end;

{ TSimpleInputForm }

procedure TSimpleInputForm.SetOnValidate(const Value: TValidationProc);
begin
  FOnValidate := Value;
end;

procedure TSimpleInputForm.Button1Click(Sender: TObject);
var
  Valid: Boolean;
begin
  Valid := True;
  if Assigned(FOnValidate) then begin
    try
      FOnValidate(ValueEdit.Text, Valid);
    except
      Valid := False;
    end;
  end;

  if Valid then
    ModalResult := mrOk
  else
    MessageDlg(SInvalidValue, mtError, [mbOk], 0);
end;

end.
